/****
 *
 * Exemplo: Lista indexada no ordenada
 *
 * Arquivo CriaArquivoBin.c: Programa que cria o arquivo binrio
 *
 * Seo: 1.2.1
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Funes de Entrada/Sada */
#include <string.h>   /* Processamento de strings */
#include <stdlib.h>   /* Funo strtod()          */
#include "ASSEGURA.h" /* Macro ASSEGURA()         */

/***************** Constantes Simblicas ******************/

   /* Nome do arquivo de texto (diretrio corrente) */
#define NOME_ARQUIVO_TXT  "Tudor.txt"

   /* Nome do arquivo binrio (diretrio corrente) */
#define NOME_ARQUIVO_BIN "Tudor.bin"

   /* Tamanho mximo de uma linha no arquivo */
#define TAMANHO_LINHA 60

   /* Nmero mximo de caracteres num nome */
#define MAX_NOME      20
   /* Nmero de caracteres numa matrcula  */
#define TAM_MATR      4

   /* Nmero mximo de elementos na lista  */
#define MAX_ELEMENTOS 20

/****************** Definies de Tipos *******************/

   /* Tipo do nome */
typedef char tNome[MAX_NOME + 1];

   /* Tipo da matrcula */
typedef char tMatricula[TAM_MATR + 1];

typedef struct {
           tNome      nome;   /* Nome do aluno */
           tMatricula matr;   /* Sua matrcula */
           double     n1, n2; /* Suas notas    */
        } tAluno;

typedef tAluno tElemento;

/***************** Definies de Funes ******************/

/****
 *
 * CriaArquivoBin(): L arquivo de texto e o armazena
 *                   num arquivo binrio
 *
 * Parmetros:
 *      arqTxt (entrada) - nome do arquivo de texto
 *      arqBin (entrada) - nome do arquivo binrio
 *
 * Retorno: Nada
 *
 ****/
void CriaArquivoBin( const char *arqTxt, const char *arqBin )
{
   char       linha[TAMANHO_LINHA + 1];
   FILE      *streamTxt, *streamBin;
   tElemento  umRegistro;
   char      *str;
   int        ocorreuErro = 0;

      /* Abre arquivo de texto em formato texto para leitura */
   streamTxt = fopen(arqTxt, "r");

      /* Se o arquivo no puder ser aberto, */
      /* nada mais pode ser feito           */
   ASSEGURA(streamTxt, "Arquivo de texto nao pode ser aberto");

      /* Abre arquivo binrio para escrita. Se o arquivo */
      /* j existir, seu contedo ser destrudo.        */
   streamBin = fopen(arqBin, "wb");

      /* Se o arquivo no puder ser aberto, */
      /* nada mais pode ser feito           */
   ASSEGURA(streamBin, "Arquivo binrio nao pode ser aberto");

      /* L cada linha do arquivo e a usa para    */
      /* construir um registro. Ento, o registro */
      /*  armazenado no arquivo binrio.         */
   while ( fgets(linha, TAMANHO_LINHA + 1, streamTxt) ) {
         /* Se ocorreu erro de leitura, encerra o lao */
      if (ferror(streamTxt)) {
         break;
      }

         /* Obtm o nome e acrescenta-o ao registro*/
      str = strtok(linha, "\t\n");
      strcpy(umRegistro.nome, str);

         /* Obtm a matrcula e acrescenta-a ao registro*/
      str = strtok(NULL, "\t\n");
      strcpy(umRegistro.matr, str);

         /* Obtm a 1a. nota e acrescenta-a  */
         /* convertida em double ao registro */
      str = strtok(NULL, "\t\n");
      umRegistro.n1 = strtod(str, NULL);

         /* Obtm a 2a. nota e acrescenta-a  */
         /* convertida em double ao registro */
      str = strtok(NULL, "\t\n");
      umRegistro.n2 = strtod(str, NULL);

      fwrite(&umRegistro, sizeof(umRegistro), 1, streamBin);

      if (ferror(streamBin)) {
         break;
      }

   }

   ocorreuErro = ferror(streamTxt) || ferror(streamBin);

      /* Processamento terminado. Fecha os arquivos. */
   fclose(streamTxt);
   fclose(streamBin);

   ASSEGURA(!ocorreuErro,
            "Ocorreu um erro de leitura ou escrita em arquivo");
}

/****
 *
 * main(): Executa a funo CriaArquivoBin()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   CriaArquivoBin(NOME_ARQUIVO_TXT, NOME_ARQUIVO_BIN);

      /* Despede-se do usurio */
   printf( "\n\t>>> Criacao do arquivo binario foi "
          "bem sucedida\n" );

   return 0;
}

